﻿define("epi-ecf-ui/contentediting/editors/_BaseEntryCollectionEditorGrid", [
// dojo
    "dojo/_base/array",
    "dojo/_base/declare",
    "dojo/aspect",
    "dojo/Deferred",
    "dojo/when",

// dijit
    "dijit/_Widget",
    "dijit/_TemplatedMixin",

// dgrid
    "dgrid/Selection",
    "dgrid/OnDemandGrid",

// epi
    "epi/dependency",
    "epi/string",
    "epi/shell/_ContextMixin",
    "epi/shell/command/builder/ButtonBuilder",
    "epi/shell/dgrid/_EditorMetadataMixin",
    "epi/shell/widget/_FocusableMixin",
    "epi/shell/widget/_ModelBindingMixin",
    "epi/shell/widget/dialog/Confirmation",

// epi cms
    "epi-cms/dgrid/DnD",
    "epi-cms/contentediting/editors/_AddItemDialogMixin",
    "epi-cms/core/ContentReference",
    "epi-cms/dgrid/WithContextMenu",
    "epi-cms/dgrid/formatters",

// epi ecf
    "../ModelSupport",
    "./_CollectionEditorDndMixin",
    "./_GridWithDropContainerMixin",
    "./_GridWithItemEditorMixin",
     "../../dgrid/_ClickablePathColumnMixin",

    "epi/i18n!epi/cms/nls/commerce.contentediting.editors.relationcollectioneditor"
],
function (
// dojo
    array,
    declare,
    aspect,
    Deferred,
    when,

// dijit
    _Widget,
    _TemplatedMixin,

// dgrid
    Selection,
    Grid,

// epi
    dependency,
    epiString,
    _ContextMixin,
    ButtonBuilder,
    _EditorMetadataMixin,
    _FocusableMixin,
    _ModelBindingMixin,
    Confirmation,

// epi cms
    DnD,
    _AddItemDialogMixin,
    ContentReference,
    WithContextMenu,
    formatters,

// epi ecf
    ModelSupport,
    _CollectionEditorDndMixin,
    _GridWithDropContainerMixin,
    _GridWithItemEditorMixin,
    _ClickablePathColumnMixin,

    resources
) {
    return declare([_Widget, _TemplatedMixin, _ModelBindingMixin, _AddItemDialogMixin, _FocusableMixin, _CollectionEditorDndMixin, _ContextMixin, _GridWithDropContainerMixin, _GridWithItemEditorMixin], {

        grid: null,

        storeKey: "epi.commerce.relation",

        itemType: "EPiServer.Commerce.Shell.Rest.Models.RelationModel",

        allowedDndTypes: [ModelSupport.contentTypeIdentifier.variationContent, ModelSupport.contentTypeIdentifier.productContent, ModelSupport.linkTypeIdentifier.relation],

        itemEditorTypes: [ModelSupport.contentTypeIdentifier.variationContent, ModelSupport.contentTypeIdentifier.packageContent, ModelSupport.contentTypeIdentifier.productContent],

        dndSourceTypes: [ModelSupport.linkTypeIdentifier.relation],

        includedColumns: ["ContentTypeIdentifier", "Name", "Path", "Quantity", "GroupName"],

        editableColumns: ["Quantity", "GroupName"],

        // Grid action column name
        cellActionName: "epiGridAction",

        noDataMessage: resources.nodatamessage,

        addLabelText: resources.addlabel,

        _store: null,

        postMixInProperties: function () {
            this.model = this.model || new this.modelType({
                itemType: this.itemType
            });
        },

        postCreate: function () {
            this.inherited(arguments);

            if (!this.roots) {
                var contentRepositoryDescriptors = dependency.resolve("epi.cms.contentRepositoryDescriptors"),
                    settings = contentRepositoryDescriptors.catalog;
                this.roots = settings.roots;
            }
            var builder = new ButtonBuilder({ settings: { showLabel: true } }),
                command = this.model.getListCommand();
            builder.create(command, this.commandTargetNode);

            // Set up metadata capable grid type
            var EditorGrid = declare([Grid, Selection, _EditorMetadataMixin, WithContextMenu, DnD, _ClickablePathColumnMixin], {});

            this._store = this._createStore();
            // Get metadata for itemType
            when(this.model.metadataManager.getMetadataForType(this.itemType), function (metadata) {
                // Set up columns object with one column for context menu
                var columns = [
                    {
                        field: "contentTypeIdentifier",
                        renderHeaderCell: function () { }, // no header
                        get: function (object) {
                            return object.contentTypeIdentifier;
                        },
                        label: "",
                        formatter: function (typeIdentifier, additionalClass) {
                            return formatters.contentIcon(typeIdentifier, additionalClass);
                        },
                        className: "epi-columnIcon16x16",
                        sortable: false
                    },
                    {
                        field: this.cellActionName,
                        renderHeaderCell: function () { }, // no header
                        renderCell: function (object, value, node, options) {
                            var builder = new ButtonBuilder({ settings: { showLabel: false } }),
                                commands = this.model.getCommands(this.grid, null);

                            array.forEach(commands, function(command) {
                                builder.create(command, node);
                            });
                        }.bind(this),
                        className: "epi-columnNarrow",
                        sortable: false
                    }],
                    store = this._store,
                    // Create grid
                    grid = this.grid = new EditorGrid({
                        store: store,
                        minWidth: 100,
                        noDataMessage: this.noDataMessage,
                        selectionMode: "single",
                        "class": "epi-plain-grid epi-plain-grid-modal epi-plain-grid--margin-bottom epi-plain-grid--cell-borders",
                        columns: columns,
                        metadata: {
                            properties: metadata.properties,
                            gridIncluded: this.includedColumns,
                            gridEditable: this.editableColumns
                        },
                        dndSourceTypes: this.dndSourceTypes,
                        dndParams: {
                            copyOnly: true,
                            accept: this.allowedDndTypes || [],
                            creator: this._dndNodeCreator.bind(this)
                        }
                    }, this.gridNode);
                this.own(grid,
                    this.model.on("itemAdded", function (e) {
                        this.grid.refresh();
                    }.bind(this)),
                    this.model.on("itemsRemoved", function (e) {
                        this.grid.refresh();
                    }.bind(this)),
                    this.model.on("itemSaved", function () {
                        this.grid.refresh();
                    }.bind(this)),
                    this.model.watch("contentLink", function (property, oldValue, newValue) {
                        // Set query even if the content link is the same, to update bundle entry grid
                        this._updateGridQuery(newValue);
                        grid.resize();
                    }.bind(this)),
                    // Listen remove command event
                    this.model.on("removeCommandEvent", function () {
                        // Show confirmation dialog to confirm delete bundle item
                        when(this._showConfirmation(this.deleteConfirmationTitle, this.deleteConfirmationDescription), function () {
                            if (this.model) {
                                var remove = [];
                                for (var selected in this.grid.selection) {
                                    if (this.grid.selection.hasOwnProperty(selected)) {
                                        remove.push(this.model.store.get(selected));
                                    }
                                }
                                return this.model.removeItems(remove);
                            }
                        }.bind(this));
                    }.bind(this))
                );
                this.styleGrid();
                this._setupDnD();
                when(this.getCurrentContext(), function(currentContext){
                    this._updateGridQuery(currentContext.id);
                    this.grid.startup();
                }.bind(this));
            }.bind(this));

        },

        _createStore: function () {
            return dependency.resolve("epi.storeregistry").get(this.storeKey);
        },

        _updateGridQuery: function(value){
            var queryOptions = this.getQueryOptions(value);
            this.grid.set("query", queryOptions.query, queryOptions.options);
        },

        getQueryOptions: function(referenceId) {
            return {
                query: {
                    referenceId: new ContentReference(referenceId).createVersionUnspecificReference().toString(),
                    relationTypes: [this.relationType]
                },
                options: {
                    ignore: ["referenceId", "relationTypes"]
                }
            };
        },

        _addItem: function (item) {
            when(this._dndGetItemData(item), function (itemData) {
                return this.model.addItem(
                    itemData,
                    this.grid.dndSource.current != null ?
                        this.grid.dndSource.getItem(this.grid.dndSource.current.id).data :
                        null,
                    this.grid.dndSource.before
                );
            }.bind(this));
        },

        resize: function () {
            this.inherited(arguments);

            if (this.grid) {
                this.grid.resize();
            }
        },

        _getDialogTitleText: function () {
            return this.addLabelText;
        },

        _dndGetItemData: function (item) {
            return when(item.data, function (data) {
                return this._createItem(data.contentLink, data.name);
            }.bind(this));
        },

        _createItem: function (targetLink, name) {
            return {
                name: name || targetLink,
                groupName: "Default",
                quantity: 1,
                sortOrder: 0,
                source: this.model.get("contentLink"),
                target: targetLink,
                type: this.relationType
            };
        },

        onExecuteDialog: function () {
            var item = this._itemEditor.get("value");
            this._addItem({ data: { contentLink: item } }, true);
        },

        _setValueAttr: function (value) {
            // summary:
            //      Value setter.
            // description:
            //      Push value to the model to be its data.
            //  tags:
            //      private

            this._set("value", value);

            if (this.model) {
                this.model.set("contentLink", value);
            }
        },

        destroy: function () {
            // summary:
            //      Destroy the object.

            if (this.grid) {
                this.grid.destroy();
                this.grid = null;
            }

            this.inherited(arguments);
        },

        _showConfirmation: function (title, description) {
            // summary:
            //      Wrap epi.shell.widget.dialog.Confirmation for short type
            //      and return deferred object
            // description:
            //      String: Text to display on dialog
            // tags:
            //      private

            var deferred = new Deferred();

            var dialog = new Confirmation({
                destroyOnHide: true,
                title: epiString.toHTML(title),
                description: epiString.toHTML(description),
                onAction: function (confirmed) {
                    if (confirmed) {
                        deferred.resolve();
                    } else {
                        deferred.cancel();
                    }
                }
            });
            dialog.show();

            return deferred.promise;
        }
    });
});
