﻿define("epi-ecf-ui/contentediting/viewmodel/PricingOverviewEditorModel", [
    // dojo
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/currency",
    "dojo/promise/all",

    // dojox
    "dojox/html/entities",

    // epi
    "epi/shell/command/DelegateCommand",
    "epi/datetime",

    // commerce
    "./_OverviewEditorModelBase",

    // resources
    "epi/i18n!epi/cms/nls/commerce.widget.pricingoverview.grid",
    "epi/i18n!epi/cms/nls/commerce.widget.pricecollection.message"
],
function (
    //dojo
    declare,
    lang,
    currency,
    all,

    // dojox
    htmlEntities,

    // epi
    DelegateCommand,
    epiDate,

    // commerce
    _OverviewEditorModelBase,

    // resources
    resources,
    res
) {
    return declare([_OverviewEditorModelBase], {
        // module:
        //      epi-ecf-ui/contentediting/viewmodel/PricingOverviewEditorModel
        // summary:
        //      Represents the model for PricingOverviewEditor

        _storeKey: "epi.commerce.price",

        _createQueryOptions: function () {
            // summary:
            //      Creates and returns query options, based on selected content link, market id and customer group (if any).
            // tags:
            //      private

            var query = { referenceId : this.get("contentLink"), query: "getprice" },
                ignoreOptions = ["query", "referenceId"];

            if (this.get("marketId")) {
                lang.mixin(query, { marketId: this.get("marketId") });
                if (query.marketId === "ALL") {
                    ignoreOptions.push("marketId");
                }
            }

            if (this.get("priceCode")) {
                lang.mixin(query, { priceCode: this.get("priceCode") });
                if (query.priceCode === "ALL"){
                    ignoreOptions.push("priceCode");
                }
            }

            return { query: query, options: { ignore: ignoreOptions } };
        },

        getCommands: function (model, category) {
            var commonSettings = {
                category: category,
                model: model,
                canExecute: true,
                isAvailable: true
            };

            return [
                new DelegateCommand(lang.mixin({
                    name: "duplicate",
                    label: resources.commands.duplicate,
                    iconClass: "epi-iconDuplicatePage",
                    delegate: function (cmd) {
                        this.emit("duplicateCommandEvent");
                    }.bind(this)
                }, commonSettings)),
                new DelegateCommand(lang.mixin({
                    name: "remove",
                    label: resources.commands.remove,
                    iconClass: "epi-iconClose",
                    delegate: function (cmd) {
                        this.emit("removeCommandEvent");
                    }.bind(this)
                }, commonSettings))
            ];
        },

        removeItems: function (models) {
            var promises = [];

            for (var i = 0, j = models.length; i < j; i += 1) {
                promises.push(this.store.remove(models[i].id));
            }

            return all(promises).then(function(removed) {
                this.emit("itemsRemoved", { removedItems: removed });
            }.bind(this));
        },

        generateFormatters: function (columnDefinitions, editableColumns) {
            for(var definitionName in columnDefinitions){
                if (!this._isColumnEditable(definitionName, editableColumns)){
                    continue;
                }
                var column = columnDefinitions[definitionName];
                if (definitionName === "validDate"){
                    column.formatter = this._validDateFormatter.bind(this);
                } else if(definitionName === "unitPrice"){
                    column.formatter = this._unitPriceFormatter.bind(this);
                } else if (definitionName === "priceCode"){
                    column.formatter = this._priceCodeFormatter.bind(this);
                } else if(column.editorArgs && column.editorArgs.selections && column.editorArgs.selections.length > 0){
                    column.formatter = this._getSelectionFormatter(column);
                } else{
                    column.formatter = this._editableValueFormatter;
                }
                column.className = column.className ? column.className + " epi-dgrid--editable" : "epi-dgrid--editable";
            }
        },

        _validDateFormatter: function(value){
            var formattedDate = this._formatValidDate(value);
            return this._editableValueFormatter(formattedDate);
        },

        _formatValidDate: function (value) {
            // Format datetime
            return this._formatDate(value.validFrom) + " - " + this._formatDate(value.validUntil);
        },

        _formatDate: function (datetime) {
            // summary:
            //      Format datetime to short type.
            // datetime: Object
            //      The raw data.
            // tags:
            //      public

            return datetime && datetime !== "" ? epiDate.toUserFriendlyString(datetime) : res.untildatenotdefined;
        },

        _unitPriceFormatter: function(value){
            var formattedCurrency = currency.format(value.amount, { currency: value.currency });
            return this._editableValueFormatter(formattedCurrency);
        },

        _priceCodeFormatter: function(value){
            var encodedValue = htmlEntities.encode(value);
            return this._editableValueFormatter(encodedValue);
        }
    });
});