﻿define("epi-ecf-ui/contentediting/viewmodel/ReadOnlyCollectionEditorModel", [
    // dojo
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/when",

    "epi/dependency",

    // epi cms
    "epi-cms/dgrid/formatters",
    "epi-cms/contentediting/editors/model/CollectionEditorModel",
    "epi-cms/_ContentContextMixin"
],
    function (
        //dojo
        declare,
        lang,
        when,

        dependency,

        // epi cms
        formatters,
        CollectionEditorModel,
        _ContentContextMixin
    ) {
        return declare([CollectionEditorModel, _ContentContextMixin], {
            // module: 
            //      epi-ecf-ui/contentediting/viewmodel/ReadOnlyCollectionEditorModel
            // summary:
            //      Represents the model for ReadOnly CollectionEditor

            // _storeKey: String
            //      Represents the key to get the store object. Implementation will set this property, to get, for example, price store, or inventory store
            _storeKey: null,

            // _store: Object
            //      Represents the REST store, to get data from.
            _store: null,

            // _contentLink: ContentReference
            //      Represents the underlay data, a content reference.
            _contentLink: null,

            // contentContext:
            //  accessMask: int
            //  isMasterLanguage: Boolean
            //      Represents if the current content link is in master language or not
            contentContext: null,

            // summary:
            //      Sets whether the editor's collection can be sorted or not.
            // value: Boolean
            //      The value true makes the collection sortable by clicking the grid's column headers, else false.
            // tags: 
            //      protected
            _enableSorting: false,

            postscript: function () {
                // summary:
                //      Post script initialization.
                // description:
                //      Set default values.
	    var registry = dependency.resolve("epi.storeregistry");
                this._store = this._store || registry.get(this._storeKey);
                this.inherited(arguments);
            },

            _preprocessResult: function (result) {
                // tags: 
                //      protected

                return result.models || result;
            },

            _dataSetter: function (value) {
                // summary:
                //      Set data property.
                // value: ContentReference
                //      Model value, which would be a ContentReference.
                // tags: 
                //      private

                // The model value is ContentReference. We need to query to the store, to get the appropriate data (for example: price list, or inventory list)
                this._contentLink = value;
                if (!value) {
                    this.data = value;
                    this._initData();

                    return;
                }

                when(this._store.query(this.createQuery(value)))
                    .then(this._preprocessResult.bind(this))
                    .then(function (processedResult) {
                        this.data = processedResult;
                        this._initData();
                    }.bind(this));

                when(this.getCurrentContent(), function (content) {
                    this.set("contentContext", {
                        accessMask: content.accessMask,
                        isMasterLanguage: content.currentLanguageBranch.isMasterLanguage
                    });
                }.bind(this));
            },

            _dataGetter: function () {
                // summary:
                //      Get data property.
                // tags: 
                //      private

                return this._contentLink;
            },

            createQuery: function (referenceId) {
                return { referenceId: referenceId };
            },

            generateFormatters: function (columnDefinitions) {
                // summary:
                //      Generate formatters for the specified column definitions.
                // columnDefinitions:
                //      The definition for the columns that should get the generated formatters.
                // tags: 
                //      override

                this.inherited(arguments);

                var typeColumn = columnDefinitions.contentTypeIdentifier;

                if (typeColumn) {
                typeColumn.formatter = function(typeIdentifier, additionalClass) {
                    return formatters.contentIcon(typeIdentifier, additionalClass);
                };
                }
                // Disable sort order in dgrid
                for (var columnName in columnDefinitions) {
                    lang.setObject("sortable", this._enableSorting, columnDefinitions[columnName]);
                }

                return columnDefinitions;
            }
        });
    });