﻿<%@ Page Language="C#" Inherits="System.Web.Mvc.ViewPage<MigrationModel>" %>

<%@ Import Namespace="EPiServer.Commerce.UI.Models.Migrate" %>
<%@ Import Namespace="EPiServer.Commerce.Internal.Migration" %>
<%@ Import Namespace="EPiServer.Shell" %>
<%@ Import Namespace="EPiServer.Framework.Web.Resources" %>
<%@ Import Namespace="System.Web.Mvc" %>
<html>
<head>
	<title>EPiServer Commerce Migration</title>
	<% if (Model.Status == MigrationStatus.InProgress) { %>
	<meta http-equiv="refresh" content="5" />
	<% } %>

	<%= Page.ClientResources("EPi.Shell.UI", new[] { ClientResourceType.Style }) %>
	<link rel="stylesheet" type="text/css" href="<%= Paths.ToClientResource("CMS", "ClientResources/epi-cms/epi.css") %>" />
	<link rel="stylesheet" type="text/css" href="<%= Paths.ToClientResource("EPiServer.Commerce.Shell", "ClientResources/epi-ecf-ui/epiCommerce.css") %>" />
</head>
<body class="Sleek">
	<div class="epi-view-container epi-view-container--scrollable">
		<h1>EPiServer Commerce Migration</h1>
		<p>
			As a part of updating EPiServer Commerce you may be required to execute migration steps in the 
                context of the web site. Below you can see pending as well as historical migration steps.
		</p>
		<div class="epi-notificationBar epi-notificationBarWithoutBorder">
			<h2>Status</h2>

			<% if (Model.CanStartMigrationJob) { %>
			<% Html.BeginForm("Run", "Migrate"); %>
			<p>
				<strong>You have pending migration steps.</strong>
			</p>
			<p>
				The migration steps cannot be undone. If you want to be able to roll back the changes you should take a backup of your databases (CMS and Commerce) before continuing.
			</p>
			<input id="exceutePendingSteps" type="submit" class="epi-chromeless epi-visibleLink" value="Execute all pending steps">
			<% Html.EndForm(); %>
			<% } %>

			<p><strong><span id="migrationStatusMessage"><%= Model.StatusMessage %></span></strong></p>
		</div>

		<% Html.BeginForm("Index", "Migrate", FormMethod.Get); %>
		<div class="Sleek">
			<button <%= Model.ShowSteps ? "class=\"epi-success dijitButtonNode" : "" %> type="submit" name="showSteps" value="true">
				Show migration steps
			</button>
			<button <%= Model.ShowLog ? "class=\"epi-success dijitButtonNode\"" : "" %> type="submit" name="showLog" value="true">
				Show log
			</button>
			<button <%= Model.ShowHelp ? "class=\"epi-success dijitButtonNode\"" : "" %> type="submit" name="showHelp" value="true">
				More information
			</button>
		</div>
		<% Html.EndForm(); %>

		<% if (Model.ShowSteps) { %>
		<table class="dgrid epi-plain-grid dgrid-row-table">
			<tr class="dgrid-header">
				<th class="dgrid-cell">Order</th>
				<th class="dgrid-cell">Done</th>
				<th class="dgrid-cell">Name</th>
				<th class="dgrid-cell">Description</th>
				<th class="dgrid-cell">Last Run</th>
			</tr>
			<%  int i = 0;
				foreach (var step in Model.Steps) { %>
			<tr class="dgrid-row <%= i++ % 2 == 0 ? "dgrid-row-even" : "dgrid-row-odd" %>">
				<td class="dgrid-cell epi-valign--top"><%: step.Order %></td>
				<td class="dgrid-cell epi-valign--top"><span class="dijitInline dijitIcon <%: step.StatusIcon %>"></span></td>
				<td class="dgrid-cell epi-valign--top"><%: step.Name %></td>
				<td class="dgrid-cell epi-valign--top"><%: step.Description %></td>
				<td class="dgrid-cell epi-valign--top"><%: step.LastRun %></td>
			</tr>
			<% } %>
		</table>
		<% } %>

		<% if (Model.ShowLog) {
				if (Model.Messages.Count() > 0) { %>
			<ul class="epi-chromeless epi-list">
				<% foreach (var message in Model.Messages) { %>
				<li><span class="epi-secondaryText"><%= message.ToHtmlString() %></span></li>
				<% } %>
			</ul>
			<% } else { %>
		<h2>No log messages</h2>
			<% } %>
		<% } %>

		<% if (Model.ShowHelp) { %>
		<h2>This can be automated</h2>
		<p>
			To let the migration steps run automatically, you can add &lt;add key=&quot;AutoMigrateEPiServer&quot; value=&quot;true&quot; /&gt; 
            to the appSettings section in web.config.
            Next time when you update your site via nuget packages, the migration process will be run automatically.
            If all steps run successfully, you'll be able to redirect to the home page. 
            Otherwise errors will be displayed in migration page and will also be logged via log4net.               
		</p>
		<% } %>
	</div>
</body>
</html>
